@@ -5,7 +5,7 @@ Crop is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2021, Free Software Foundation, Inc.
+Copyright (C) 2016-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -166,6 +166,19 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET,
       gal_options_parse_csv_float64
     },
+    {
+      "widthinpix",
+      UI_KEY_WIDTHINPIX,
+      0,
+      0,
+      "--width is in pixels (even in WCS mode).",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &p->widthinpix,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
     {
       "center",
       UI_KEY_CENTER,
@@ -270,9 +283,9 @@ struct argp_option program_options[] =
     {
       "polygon",
       UI_KEY_POLYGON,
-      "STR",
+      "FLT,FLT[:...]",
       0,
-      "Polygon vertices of region to crop, keep inside.",
+      "Polygon vertices, also a DS9 region file.",
       UI_GROUP_REGION,
       &p->polygon,
       GAL_TYPE_STRING,
