@@ -5,7 +5,7 @@ Crop is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2020, Free Software Foundation, Inc.
+Copyright (C) 2015-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -524,17 +524,9 @@ onecrop_make_array(struct onecropparams *crp, long *fpixel_i,
   char **strarr, cpname[FLEN_KEYWORD];
   gal_data_t *rkey=gal_data_array_calloc(1);
   size_t i, ndim=crp->p->imgs->ndim, totsize;
-  char *cp, *cpf, blankrec[80], titlerec[80];
   struct inputimgs *img=&crp->p->imgs[crp->in_ind];
 
 
-  /* Set the last element of the blank array. */
-  cpf=blankrec+79;
-  *cpf='\0';
-  titlerec[79]='\0';
-  cp=blankrec; do *cp=' '; while(++cp<cpf);
-
-
   /* Set the size of the output, in WCS mode, noblank==0. */
   if(crp->p->noblank && crp->p->mode==IMGCROP_MODE_IMG)
     for(i=0;i<ndim;++i)
@@ -554,8 +546,11 @@ onecrop_make_array(struct onecropparams *crp, long *fpixel_i,
      generally everything from 1). */
   if(fits_create_file(&ofp, outname, &status))
     gal_fits_io_error(status, "creating file");
-  fits_create_img(ofp, SHORT_IMG, 0, naxes, &status);
-  fits_close_file(ofp, &status);
+  if(crp->p->primaryimghdu==0)
+    {
+      fits_create_img(ofp, SHORT_IMG, 0, naxes, &status);
+      fits_close_file(ofp, &status);
+    }
 
 
   /* Create the output crop image. */
@@ -610,15 +605,7 @@ onecrop_make_array(struct onecropparams *crp, long *fpixel_i,
   if(img->wcs)
     {
       /* Write the WCS title and common WCS information. */
-      if(fits_write_record(ofp, blankrec, &status))
-        gal_fits_io_error(status, NULL);
-      sprintf(titlerec, "%sWCS information", GAL_FITS_KEY_TITLE_START);
-      for(i=strlen(titlerec);i<79;++i)
-        titlerec[i]=' ';
-      fits_write_record(ofp, titlerec, &status);
-      for(i=0;i<img->nwcskeys-1;++i)
-        fits_write_record(ofp, &img->wcstxt[i*80], &status);
-      gal_fits_io_error(status, NULL);
+      gal_fits_key_write_wcsstr(ofp, img->wcs, img->wcstxt, img->nwcskeys);
 
       /* Correct the CRPIX keywords. */
       for(i=0;i<ndim;++i)
@@ -631,12 +618,10 @@ onecrop_make_array(struct onecropparams *crp, long *fpixel_i,
     }
 
 
-  /* Add the Crop information. */
-  sprintf(titlerec, "%sCrop information", GAL_FITS_KEY_TITLE_START);
-  for(i=strlen(titlerec);i<79;++i)
-    titlerec[i]=' ';
-  if(fits_write_record(ofp, titlerec, &status))
-    gal_fits_io_error(status, NULL);
+  /* Add the Crop information title, the actual keywords will be written
+     later. The keywords will contain the pixels of the is cropped region
+     from input image(s): */
+  gal_fits_key_write_title_in_ptr("Crop information", ofp);
 }
 
 
@@ -752,7 +737,8 @@ onecrop(struct onecropparams *crp)
       /* A section has been added to the cropped image from this input
          image, so save the information of this image. */
       sprintf(basekeyname, "ICF%zu", crp->numimg);
-      gal_fits_key_write_filename(basekeyname, img->name, &headers, 0);
+      gal_fits_key_write_filename(basekeyname, img->name, &headers, 0,
+                                  p->cp.quiet);
       sprintf(regionkey, "%sPIX", basekeyname);
       gal_fits_key_list_add_end(&headers, GAL_TYPE_STRING, regionkey,
                                 0, region, 0, "Range of pixels used for "
