@@ -5,7 +5,7 @@ Fits is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2021, Free Software Foundation, Inc.
+Copyright (C) 2016-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -172,14 +172,18 @@ parse_opt(int key, char *arg, struct argp_state *state)
   /* Set the key to this option. */
   switch(key)
     {
-
-    /* Read the non-option tokens (arguments): */
+    /* Read the non-option tokens (arguments). */
     case ARGP_KEY_ARG:
-      /* Only FITS files are acceptable. */
-      if( gal_fits_name_is_fits(arg) )
-        gal_list_str_add(&p->input, arg, 1);
-      else
-        argp_error(state, "%s is not a recognized FITS file", arg);
+      /* The user may give a shell variable that is empty! In that case
+         'arg' will be an empty string! We don't want to account for such
+         cases (and give a clear error that no input has been given). */
+      if(arg[0]!='\0')
+        {
+          if( gal_fits_file_recognized(arg) )
+            gal_list_str_add(&p->input, arg, 1);
+          else
+            argp_error(state, "%s is not a recognized FITS file", arg);
+        }
       break;
 
 
@@ -324,7 +328,7 @@ ui_read_check_only_options(struct fitsparams *p)
      mode flag to keyword-mode. */
   if( p->date || p->comment || p->history || p->asis || p->keyvalue
       || p->delete || p->rename || p->update || p->write || p->verify
-      || p->printallkeys || p->copykeys || p->datetosec
+      || p->printallkeys || p->printkeynames || p->copykeys || p->datetosec
       || p->wcscoordsys || p->wcsdistortion )
     {
       /* Check if a HDU is given. */
@@ -340,6 +344,7 @@ ui_read_check_only_options(struct fitsparams *p)
       /* Keyword-related options that must be called alone. */
       checkkeys = ( (p->keyvalue!=NULL)
                     + (p->datetosec!=NULL)
+                    + (p->printkeynames!=0)
                     + (p->wcscoordsys!=NULL)
                     + (p->wcsdistortion!=NULL) );
       if( ( checkkeys
@@ -361,13 +366,19 @@ ui_read_check_only_options(struct fitsparams *p)
       if(p->wcsdistortion)
         p->distortionid=gal_wcs_distortion_from_string(p->wcsdistortion);
 
+      /* Make sure the value of '--keyvalue' is actually present. */
+      if(p->keyvalue && p->keyvalue->v[0]=='\0')
+        error(EXIT_FAILURE, 0, "the '--keyvalue' option requires a value: "
+              "the name(s) of keywords you want the value of");
+
       /* Set the operating mode. */
       p->mode=FITS_MODE_KEY;
     }
 
   /* Same for the extension-related options */
   if( p->remove || p->copy || p->cut || p->numhdus || p->datasum
-      || p->pixelscale || p->skycoverage )
+      || p->pixelscale || p->skycoverage || p->hastablehdu
+      || p->hasimagehdu || p->listtablehdus || p->listimagehdus )
     {
       /* A small sanity check. */
       if(p->mode!=FITS_MODE_INVALID)
@@ -375,26 +386,32 @@ ui_read_check_only_options(struct fitsparams *p)
               "cannot be called together");
 
       /* Some HDU options cannot be called with other options. */
-      stdoutcheck = p->numhdus + p->datasum + p->pixelscale + p->skycoverage;
+      stdoutcheck = ( p->numhdus + p->datasum + p->pixelscale
+                      + p->skycoverage + p->hastablehdu + p->hasimagehdu
+                      + p->listtablehdus + p->listimagehdus );
 
       /* Make sure if an output file is needed. */
       if(stdoutcheck)
         {
-          /* Make sure the other HDU-related options aren't called. */
+          /* Make sure HDU reading and editing options aren't called
+             together. */
           if(p->remove || p->copy || p->cut)
-            error(EXIT_FAILURE, 0, "'--numhdus', '--datasum', '--pixelscale' "
-                  "or '--skycoverage' options cannot be called with any of "
-                  "the '--remove', '--copy' or '--cut' options");
+            error(EXIT_FAILURE, 0, "HDU reading options (like "
+                  "'--numhdus', '--datasum' and etc) cannot be called "
+                  "with any of the HDU modification options like "
+                  "'--remove', '--copy' or '--cut' options");
 
           /* Make sure these options are called alone. */
           if(stdoutcheck>1)
-            error(EXIT_FAILURE, 0, "'--numhdus', '--datasum', '--pixelscale' "
-                  "or '--skycoverage' options cannot be called together, "
-                  "only one at a time");
+            error(EXIT_FAILURE, 0, "HDU info options, like '--numhdus', "
+                  "'--datasum', '--pixelscale' or '--skycoverage', cannot "
+                  "be called together, only one at a time");
 
           /* Make sure the HDU is given if any of the options except
              '--numhdus' are called. */
-          if( stdoutcheck-p->numhdus && p->cp.hdu==NULL )
+          if( ( p->numhdus || p->hastablehdu || p->hasimagehdu
+                || p->listtablehdus || p->listimagehdus)
+              && p->cp.hdu==NULL )
             error(EXIT_FAILURE, 0, "a HDU (extension) is necessary for the "
                   "'--datasum', '--pixelscale' or '--skycoverage' options. "
                   "Please use the '--hdu' (or '-h') option to select one");
