@@ -5,7 +5,7 @@ Table is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2021, Free Software Foundation, Inc.
+Copyright (C) 2016-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -101,7 +101,7 @@ struct argp_option program_options[] =
       UI_KEY_CATCOLUMNS,
       "STR",
       0,
-      "Columns to use in catcolumnfile.",
+      "Columns to use in --catcolumnfile.",
       GAL_OPTIONS_GROUP_INPUT,
       &p->catcolumns,
       GAL_TYPE_STRLL,
@@ -109,6 +109,32 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "catrowfile",
+      UI_KEY_CATROWFILE,
+      "FITS/TXT",
+      0,
+      "File(s) to be concatenated by row.",
+      GAL_OPTIONS_GROUP_INPUT,
+      &p->catrowfile,
+      GAL_TYPE_STRLL,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "catrowhdu",
+      UI_KEY_CATROWHDU,
+      "STR/INT",
+      0,
+      "HDU/Extension(s) in --catrowfile.",
+      GAL_OPTIONS_GROUP_INPUT,
+      &p->catrowhdu,
+      GAL_TYPE_STRLL,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
 
 
 
@@ -224,9 +250,9 @@ struct argp_option program_options[] =
     {
       "polygon",
       UI_KEY_POLYGON,
-      "FLT:FLT[,...]",
+      "FLT,FLT[:...]",
       0,
-      "Polygon for '--inpolygon' or '--outpolygon'.",
+      "Polygon vertices, also a DS9 region file.",
       UI_GROUP_OUTROWS,
       &p->polygon,
       GAL_TYPE_STRING,
@@ -294,7 +320,7 @@ struct argp_option program_options[] =
       UI_KEY_HEAD,
       "INT",
       0,
-      "Only output given number of top rows.",
+      "Only return given number of top rows.",
       UI_GROUP_OUTROWS,
       &p->head,
       GAL_TYPE_SIZE_T,
@@ -307,7 +333,7 @@ struct argp_option program_options[] =
       UI_KEY_TAIL,
       "INT",
       0,
-      "Only output given number of bottom rows.",
+      "Only return given number of bottom rows.",
       UI_GROUP_OUTROWS,
       &p->tail,
       GAL_TYPE_SIZE_T,
@@ -362,12 +388,24 @@ struct argp_option program_options[] =
       0,
       "Remove rows with blank in given columns.",
       GAL_OPTIONS_GROUP_INPUT,
-      &p->noblank,
-      GAL_TYPE_STRING,
+      &p->noblankll,
+      GAL_TYPE_STRLL,
       GAL_OPTIONS_RANGE_ANY,
       GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET,
-      gal_options_parse_csv_strings
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "noblankend",
+      UI_KEY_NOBLANKEND,
+      "STR[,STR]",
+      0,
+      "Sim. to --noblank, at end (e.g., after arith.).",
+      GAL_OPTIONS_GROUP_INPUT,
+      &p->noblankend,
+      GAL_TYPE_STRLL,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
     },
 
 
