@@ -5,7 +5,7 @@ Table is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2021, Free Software Foundation, Inc.
+Copyright (C) 2016-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -28,6 +28,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <stdio.h>
 #include <string.h>
 
+#include <gnuastro/ds9.h>
 #include <gnuastro/wcs.h>
 #include <gnuastro/fits.h>
 #include <gnuastro/table.h>
@@ -180,16 +181,17 @@ parse_opt(int key, char *arg, struct argp_state *state)
   /* Set the key to this option. */
   switch(key)
     {
-
     /* Read the non-option tokens (arguments): */
     case ARGP_KEY_ARG:
+      /* The user may give a shell variable that is empty! In that case
+         'arg' will be an empty string! We don't want to account for such
+         cases (and give a clear error that no input has been given). */
       if(p->filename)
         argp_error(state, "only one argument (input file) should be given");
       else
-        p->filename=arg;
+        if(arg[0]!='\0') p->filename=arg;
       break;
 
-
     /* This is an option, set its value. */
     default:
       return gal_options_set_from_key(key, arg, p->cp.poptions, &p->cp);
@@ -289,6 +291,11 @@ ui_read_check_only_options(struct tableparams *p)
         error(EXIT_FAILURE, 0, "at most three values can be given to each "
               "call of '--colmetadata' ('-m') after the original columns "
               "name or number. But %zu strings have been given", tmp->size);
+
+  /* '--head' and '--tail' shouldn't be called together! */
+  if(p->head!=GAL_BLANK_SIZE_T && p->tail!=GAL_BLANK_SIZE_T)
+    error(EXIT_FAILURE, 0, "'--head' and '--tail' can't be called at "
+          "the same time");
 }
 
 
@@ -302,7 +309,7 @@ ui_check_options_and_arguments(struct tableparams *p)
      a HDU is also given. */
   if(p->filename)
     {
-      if( gal_fits_name_is_fits(p->filename) && p->cp.hdu==NULL )
+      if( gal_fits_file_recognized(p->filename) && p->cp.hdu==NULL )
         error(EXIT_FAILURE, 0, "no HDU specified. When the input is a FITS "
               "file, a HDU must also be specified, you can use the '--hdu' "
               "('-h') option and give it the HDU number (starting from "
@@ -728,6 +735,64 @@ ui_check_select_sort_read_col_ind(char *string)
 
 
 
+/* Fill replace '--noblank' with all columns (if it is given '_all'). */
+static void
+ui_noblank_prepare(struct tableparams *p, size_t numcols)
+{
+  size_t i, zero=0;
+  gal_list_str_t *tmp;
+  char name[50]; /* No table will have a 50 digit number of columns! */
+
+  /* Merge all the possible calls of 'noblank' into one. */
+  gal_options_merge_list_of_csv(&p->noblankll);
+
+  /* Initialize the 'noblank' list of (empty!) datasets (we only need the
+     column names, this is done to conform with the general selection
+     mechanism, where some selection methods need values). */
+  p->noblank=NULL;
+
+  /* In case it is given an '_all' option (for all columns to be
+     checked). */
+  if(gal_list_str_number(p->noblankll)==1
+     && !strcmp(p->noblankll->v,"_all"))
+    {
+      /* Add all the columns to the final list of datasets. */
+      for(i=0;i<numcols;++i)
+        {
+          sprintf(name, "%zu", i+1);
+          gal_list_data_add_alloc(&p->noblank, NULL, GAL_TYPE_FLOAT64, 1,
+                                  &zero, NULL, 0, -1, 1, name, NULL, NULL);
+        }
+    }
+
+  /* Only certain columns are requested (we should just put the strings in
+     the desired format of a list of datasets with names, not a single
+     dataset with type string and many elements). */
+  else
+    {
+      for(tmp=p->noblankll; tmp!=NULL; tmp=tmp->next)
+        gal_list_data_add_alloc(&p->noblank, NULL, GAL_TYPE_FLOAT64, 1,
+                                &zero, NULL, 0, -1, 1, tmp->v, NULL, NULL);
+    }
+
+  /* Reverse the final output list of columns. */
+  gal_list_data_reverse(&p->noblank);
+
+  /* Clean up the list that we don't need any more. */
+  gal_list_str_free(p->noblankll, 1);
+  p->noblankll=NULL;
+
+  /* For a check:
+  gal_data_t *dtmp;
+  for(dtmp=p->noblank;dtmp!=NULL;dtmp=dtmp->next)
+    printf("col: %s\n", dtmp->name);
+  exit(0);
+  */
+}
+
+
+
+
 
 /* See if row selection or sorting needs any extra columns to be read. */
 static void
@@ -747,10 +812,11 @@ ui_check_select_sort_before(struct tableparams *p, gal_list_str_t *lines,
   gal_data_t *dtmp, *allcols, *inpolytmp=NULL, *outpolytmp=NULL;
 
   /* Important note: these have to be in the same order as the 'enum
-     select_types' in 'main.h'. We'll fill the two polygon elements
-     later. */
+     select_types' in 'main.h'. We'll fill the NULL components
+     afterwards. */
   gal_data_t *select[SELECT_TYPE_NUMBER]={p->range, NULL, NULL,
-                                          p->equal, p->notequal};
+                                          p->equal, p->notequal,
+                                          NULL};
 
 
   /* The inpolygon dataset is currently a single dataset with two elements
@@ -776,14 +842,32 @@ ui_check_select_sort_before(struct tableparams *p, gal_list_str_t *lines,
     }
 
 
+  /* Get all the input table's column information. */
+  allcols=gal_table_info(p->filename, p->cp.hdu, lines, &numcols,
+                         &numrows, &tableformat);
+
+
+  /* If '--noblank' is given an '_all', we should replace it with the
+     counters of all the input table columns. */
+  if(p->noblankll)
+    {
+      ui_noblank_prepare(p, numcols);
+      select[SELECT_TYPE_NOBLANK]=p->noblank;
+    }
+
+
   /* Allocate necessary spaces. */
   if(p->selection)
     {
+      /* Find the number of columns that are needed for selection. */
       *nselect = ( gal_list_data_number(p->range)
                    + gal_list_data_number(inpolytmp)
                    + gal_list_data_number(outpolytmp)
                    + gal_list_data_number(p->equal)
-                   + gal_list_data_number(p->notequal) );
+                   + gal_list_data_number(p->notequal)
+                   + gal_list_data_number(p->noblank) );
+
+      /* Allocate the necessary arrays. */
       selectind=gal_pointer_allocate(GAL_TYPE_SIZE_T, *nselect, 0,
                                      __func__, "selectind");
       selecttype=gal_pointer_allocate(GAL_TYPE_SIZE_T, *nselect, 0,
@@ -792,7 +876,12 @@ ui_check_select_sort_before(struct tableparams *p, gal_list_str_t *lines,
                                         __func__, "selectindout");
       selecttypeout=gal_pointer_allocate(GAL_TYPE_SIZE_T, *nselect, 0,
                                          __func__, "selecttypeout");
+
+      /* Initialize them to blank. */
       sf=(s=selectindout)+*nselect; do *s++=GAL_BLANK_SIZE_T; while(s<sf);
+
+      /* Set the output pointers (they have already been initialized to
+         NULL by the caller function). */
       *selectindout_out=selectindout;
       *selecttypeout_out=selecttypeout;
     }
@@ -811,11 +900,6 @@ ui_check_select_sort_before(struct tableparams *p, gal_list_str_t *lines,
         }
 
 
-  /* Get all the column information. */
-  allcols=gal_table_info(p->filename, p->cp.hdu, lines, &numcols,
-                         &numrows, &tableformat);
-
-
   /* If the values are numbers, we'll check if the given value is less than
      the total number of columns. Just note that the indexs count from
      zero. */
@@ -1071,7 +1155,7 @@ ui_preparations(struct tableparams *p)
 
   /* If any kind of row-selection is requested set 'p->selection' to 1. */
   p->selection = ( p->range || p->inpolygon || p->outpolygon || p->equal
-                   || p->notequal );
+                   || p->notequal || p->noblankll );
 
 
   /* If row sorting or selection are requested, see if we should read any
@@ -1092,8 +1176,8 @@ ui_preparations(struct tableparams *p)
 
   /* Read the necessary columns. */
   p->table=gal_table_read(p->filename, cp->hdu, lines, p->columns,
-                          cp->searchin, cp->ignorecase, cp->minmapsize,
-                          p->cp.quietmmap, colmatch);
+                          cp->searchin, cp->ignorecase, cp->numthreads,
+                          cp->minmapsize, p->cp.quietmmap, colmatch);
   if(p->filename==NULL) p->filename="stdin";
   gal_list_str_free(lines, 1);
 
@@ -1117,12 +1201,6 @@ ui_preparations(struct tableparams *p)
     arithmetic_indexs_final(p, colmatch);
 
 
-  /* Now that the data columns are ready, we can free the string linked
-     list. */
-  gal_list_str_free(p->columns, 1);
-  p->columns=NULL;
-
-
   /* Make sure the (possible) output name is writable. */
   gal_checkset_writable_remove(p->cp.output, 0, p->cp.dontdelete);
 
@@ -1263,6 +1341,7 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct tableparams *p)
   /* Read/allocate all the necessary starting arrays. */
   ui_preparations(p);
 
+
   /* Let the user know basic information if necessary (for example when a
      random number generator has been used). */
   if(p->rng && !p->cp.quiet)
@@ -1307,8 +1386,13 @@ ui_free_report(struct tableparams *p)
   free(p->cp.output);
   ui_outcols_free(p->outcols);
   gal_list_data_free(p->table);
-  gal_list_data_free(p->colmetadata);
+  if(p->wcshdu) free(p->wcshdu);
+  gal_list_data_free(p->noblank);
+  gal_list_str_free(p->columns, 1);
   if(p->colarray) free(p->colarray);
+  gal_list_data_free(p->colmetadata);
+  gal_list_str_free(p->catcolumnhdu, 1);
+  gal_list_str_free(p->catcolumnfile, 1);
 
   /* If a random number generator was allocated, free it. */
   if(p->rng) gsl_rng_free(p->rng);
