@@ -6,7 +6,8 @@ Original author:
      Kartik Ohri <kartikohri13@gmail.com>
 Contributing author(s):
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
-Copyright (C) 2020-2021, Free Software Foundation, Inc.
+     Pedram Ashofteh Ardakani <pedramardakani@pm.me>
+Copyright (C) 2020-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -30,6 +31,8 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <stdlib.h>
 #include <string.h>
 
+#include <gsl/gsl_math.h>
+
 #include <gnuastro/type.h>
 #include <gnuastro/pointer.h>
 
@@ -263,7 +266,7 @@ gal_units_degree_to_ra(double decimal, int usecolon)
   /* Format the extracted hours, minutes and seconds as a string with
      leading zeros if required, in hh:mm:ss format */
   nchars = snprintf(ra, UNITS_RADECSTR_MAXLENGTH-1,
-                    usecolon ? "%02d:%02d:%g" : "%02dh%02dm%gs",
+                    usecolon ? "%02d:%02d:%g" : "%02dh%02dm%g",
                     hours, minutes, seconds);
   if(nchars>UNITS_RADECSTR_MAXLENGTH)
     error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to address "
@@ -321,7 +324,7 @@ gal_units_degree_to_dec(double decimal, int usecolon)
      with leading zeros if required, in hh:mm:ss format with correct
      sign. */
   nchars = snprintf(dec, UNITS_RADECSTR_MAXLENGTH-1,
-                    usecolon ? "%s%02d:%02d:%g" : "%s%02dd%02dm%gs",
+                    usecolon ? "%s%02d:%02d:%g" : "%s%02dd%02dm%g",
                     sign<0?"-":"+", degrees, arc_minutes, arc_seconds);
   if(nchars>UNITS_RADECSTR_MAXLENGTH)
     error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to address "
@@ -368,6 +371,16 @@ gal_units_counts_to_mag(double counts, double zeropoint)
 
 
 
+/* Convert magnitude to counts using the given zeropoint. */
+double
+gal_units_mag_to_counts(double mag, double zeropoint)
+{
+  return pow(10, (mag - zeropoint)/(-2.5f));
+}
+
+
+
+
 /* Convert Pixel values to Janskys with an AB-magnitude based
    zero-point. See the "Brightness, Flux, Magnitude and Surface
    brightness". */
@@ -376,3 +389,99 @@ gal_units_counts_to_jy(double counts, double zeropoint_ab)
 {
   return counts * 3631 * pow(10, -1 * zeropoint_ab / 2.5);
 }
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+/**********************************************************************/
+/****************         Distance conversions        *****************/
+/**********************************************************************/
+/* Convert Astronomical Units (AU) to Parsecs (PC). From the definition of
+   Parsecs, 648000/pi AU = 1 PC. The mathematical constant 'PI' is imported
+   from the GSL as M_PI. So: */
+double
+gal_units_au_to_pc(double au)
+{
+  return au / 648000.0f * M_PI;
+}
+
+
+
+
+
+/* Convert Parsecs (PC) to Astronomical units (AU), see comment of
+   'gal_units_au_to_pc'. */
+double
+gal_units_pc_to_au(double au)
+{
+  return au * 648000.0f / M_PI;
+}
+
+
+
+
+
+/* Convert Light-years to Parsecs, according to
+   https://en.wikipedia.org/wiki/Light-year#Definitions:
+
+   1 light-year = 9460730472580800 metres (exactly)
+                ~ 9.461 petametres
+                ~ 9.461 trillion kilometres (5.879 trillion miles)
+                ~ 63241.077 astronomical units
+                ~ 0.306601 parsecs  */
+double
+gal_units_ly_to_pc(double ly)
+{
+  return ly * 0.306601f;
+}
+
+
+
+
+
+/* Convert Parsecs to Light-years (see comment of gal_units_ly_to_pc). */
+double
+gal_units_pc_to_ly(double pc)
+{
+  return pc / 0.306601f;
+}
+
+
+
+
+
+/* Convert Astronomical Units to Light-years (see comment of
+   gal_units_ly_to_pc). */
+double
+gal_units_au_to_ly(double au)
+{
+  return au / 63241.077f;
+}
+
+
+
+
+
+/* Convert Light-years to Astronomical Units (see comment of
+   gal_units_ly_to_pc). */
+double
+gal_units_ly_to_au(double ly)
+{
+  return ly * 63241.077f;
+}
